@@ -151,10 +151,14 @@
     Updated by      Richard Gooch   26-APR-2000: Added NIS configuration
   support.
 
-    Last updated by Richard Gooch   27-APR-2000: Fixed DIR leak. Generate
+    Updated by      Richard Gooch   27-APR-2000: Fixed DIR leak. Generate
   synthetic REGISTER events on SIGHUP. Silently ignore NIS attempts when no
   NIS domain set.
 
+    Last updated by Richard Gooch   30-APR-2000: Moved parts of <action_compat>
+  to <get_old_name> in compat_name.c. Simplified <action_compat> for SCSI and
+  IDE new compatibilty entries.
+
 
 */
 #include <unistd.h>
@@ -175,7 +179,6 @@
 #include <fcntl.h>
 #include <linux/devfs_fs.h>
 #include <linux/kdev_t.h>
-#include <linux/major.h>
 #include <syslog.h>
 #include <signal.h>
 #include <regex.h>
@@ -187,15 +190,7 @@
 #include "devfsd.h"
 
 
-#ifndef IDE6_MAJOR        /*  In case we're building with an ancient kernel  */
-#  define IDE6_MAJOR      88
-#  define IDE7_MAJOR      89
-#  define IDE8_MAJOR      90
-#  define IDE9_MAJOR      91
-#endif
-
-
-#define DEVFSD_VERSION "1.3.7"
+#define DEVFSD_VERSION "1.3.8"
 #define CONFIG_FILE "/etc/devfsd.conf"
 #define MAX_ARGS (6 + 1)
 
@@ -294,6 +289,9 @@ EXTERN_FUNCTION (flag st_expr_expand,
 		  CONST char *(*get_variable) (CONST char *variable,
 					       void *info),
 		  void *info, FILE *errfp) );
+EXTERN_FUNCTION (const char *get_old_name,
+		 (const char *devname, unsigned int namelen,
+		  char *buffer, unsigned int major, unsigned int minor) );
 
 
 /*  Public data  */
@@ -1219,35 +1217,6 @@ static void action_copy (const struct devfsd_notify_struct *info,
     SYSLOG (LOG_ERR, "error copying: \"%s\" to \"%s\"\n", source, destination);
 }   /*  End Function action_copy  */
 
-struct translate_struct
-{
-    char *match;    /*  The string to match to (up to length)                */
-    char *format;   /*  Format of output, "%s" takes data past match string,
-			NULL is effectively "%s" (just more efficient)       */
-};
-
-static struct translate_struct translate_table[] =
-{
-    {"sound/",     NULL},
-    {"printers/",  "lp%s"},
-    {"v4l/",       NULL},
-    {"parports/",  "parport%s"},
-    {"fb/",        "fb%s"},
-    {"netlink/",   NULL},
-    {"loop/",      "loop%s"},
-    {"floppy/",    "fd%s"},
-    {"rd/",        "ram%s"},
-    {"md/",        "md%s"},
-    {"cua/",       "cua%s"},
-    {"tts/",       "ttyS%s"},
-    {"ip2/",       "ip2%s"},
-    {"ttf/",       "ttyF%s"},
-    {"cuf/",       "cuf%s"},
-    {"vc/",        "tty%s"},
-    {"misc/",      NULL},
-    {NULL,         NULL}
-};
-
 static void action_compat (const struct devfsd_notify_struct *info,
 			   unsigned int action)
 /*  [SUMMARY] Process a compatibility request.
@@ -1259,9 +1228,7 @@ static void action_compat (const struct devfsd_notify_struct *info,
     const char *compat_name = NULL;
     const char *dest_name = info->devname;
     char *ptr;
-    struct translate_struct *trans;
-    char compat_buf[STRING_LENGTH];
-    char dest_buf[STRING_LENGTH];
+    char compat_buf[STRING_LENGTH], dest_buf[STRING_LENGTH];
     static char function_name[] = "action_compat";
 
     /*  First construct compatibility name  */
@@ -1269,373 +1236,83 @@ static void action_compat (const struct devfsd_notify_struct *info,
     {
       case AC_MKOLDCOMPAT:
       case AC_RMOLDCOMPAT:
-	for (trans = translate_table; trans->match != NULL; ++trans)
-	{
-	    size_t len = strlen (trans->match);
-
-	    if (strncmp (info->devname, trans->match, len) == 0)
-	    {
-		if (trans->format == NULL) compat_name = info->devname + len;
-		else
-		{
-		    sprintf (compat_buf, trans->format, info->devname + len);
-		    compat_name = compat_buf;
-		}
-		break;
-	    }
-	}
-	if (compat_name != NULL) break;
-	if (strncmp (info->devname, "sbp/", 4) == 0)
-	{
-	    sprintf (compat_buf, "sbpcd%u", info->minor);
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strcmp (info->devname + info->namelen - 7,"generic") == 0) )
-	{
-	    sprintf (compat_buf, "sg%u", info->minor);
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strncmp (ptr = (strrchr (info->devname, '/') + 1), "mt", 2)
-		   == 0) )
-	{
-	    char mode = ptr[2];
-
-	    if (mode == 'n') mode = '\0';
-	    sprintf (compat_buf, "nst%u%c", info->minor & 0x1f, mode);
-	    compat_name = compat_buf;
-	    if (info->devname[info->namelen - 1] != 'n') ++compat_name;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strncmp (ptr = (strrchr (info->devname, '/') + 1), "mt", 2)
-		   == 0) )
-	{
-	    sprintf (compat_buf, "%sht%d", ptr + 2, info->minor & 0x7f);
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strcmp (info->devname + info->namelen - 2, "cd") == 0) )
-	{
-	    sprintf (compat_buf, "sr%u", info->minor);
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strcmp (info->devname + info->namelen - 4, "disc") == 0) )
-	{
-	    sprintf ( compat_buf, "sd%c", 'a' + (info->minor >> 4) );
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strncmp (ptr = (strrchr (info->devname, '/') +1), "part", 4)
-		   == 0) )
-	{
-	    sprintf (compat_buf, "sd%c%s", 'a' + (info->minor >> 4), ptr + 4);
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strcmp (info->devname + info->namelen - 4, "disc") == 0) )
-	{
-	    char letter;
-
-	    switch (info->major)
-	    {
-	      case IDE0_MAJOR:
-		letter = 'a';
-		break;
-	      case IDE1_MAJOR:
-		letter = 'c';
-		break;
-	      case IDE2_MAJOR:
-		letter = 'e';
-		break;
-	      case IDE3_MAJOR:
-		letter = 'g';
-		break;
-	      case IDE4_MAJOR:
-		letter = 'i';
-		break;
-	      case IDE5_MAJOR:
-		letter = 'k';
-		break;
-	      case IDE6_MAJOR:
-		letter = 'm';
-		break;
-	      case IDE7_MAJOR:
-		letter = 'o';
-		break;
-	      case IDE8_MAJOR:
-		letter = 'q';
-		break;
-	      case IDE9_MAJOR:
-		letter = 's';
-		break;
-	      default:
-		letter = 'y';
-		break;
-	    }
-	    if (info->minor > 63) ++letter;
-	    sprintf (compat_buf, "hd%c", letter);
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strncmp (ptr = (strrchr (info->devname, '/') +1), "part", 4)
-		   == 0) )
-	{
-	    char letter;
-
-	    switch (info->major)
-	    {
-	      case IDE0_MAJOR:
-		letter = 'a';
-		break;
-	      case IDE1_MAJOR:
-		letter = 'c';
-		break;
-	      case IDE2_MAJOR:
-		letter = 'e';
-		break;
-	      case IDE3_MAJOR:
-		letter = 'g';
-		break;
-	      case IDE4_MAJOR:
-		letter = 'i';
-		break;
-	      case IDE5_MAJOR:
-		letter = 'k';
-		break;
-	      case IDE6_MAJOR:
-		letter = 'm';
-		break;
-	      case IDE7_MAJOR:
-		letter = 'o';
-		break;
-	      case IDE8_MAJOR:
-		letter = 'q';
-		break;
-	      case IDE9_MAJOR:
-		letter = 's';
-		break;
-	      default:
-		letter = 'y';
-		break;
-	    }
-	    if (info->minor > 63) ++letter;
-	    sprintf (compat_buf, "hd%c%s", letter, ptr + 4);
-	    compat_name = compat_buf;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strcmp (info->devname + info->namelen - 2, "cd") == 0) )
-	{
-	    char letter;
-
-	    switch (info->major)
-	    {
-	      case IDE0_MAJOR:
-		letter = 'a';
-		break;
-	      case IDE1_MAJOR:
-		letter = 'c';
-		break;
-	      case IDE2_MAJOR:
-		letter = 'e';
-		break;
-	      case IDE3_MAJOR:
-		letter = 'g';
-		break;
-	      case IDE4_MAJOR:
-		letter = 'i';
-		break;
-	      case IDE5_MAJOR:
-		letter = 'k';
-		break;
-	      case IDE6_MAJOR:
-		letter = 'm';
-		break;
-	      case IDE7_MAJOR:
-		letter = 'o';
-		break;
-	      case IDE8_MAJOR:
-		letter = 'q';
-		break;
-	      case IDE9_MAJOR:
-		letter = 's';
-		break;
-	      default:
-		letter = 'y';
-		break;
-	    }
-	    if (info->minor > 63) ++letter;
-	    sprintf (compat_buf, "hd%c", letter);
-	    compat_name = compat_buf;
-	}
-	else if (strncmp (info->devname, "vcc/", 4) == 0)
-	{
-	    sprintf (compat_buf, "vcs%s", info->devname + 4);
-	    if (compat_buf[3] == '0') compat_buf[3] = '\0';
-	    compat_name = compat_buf;
-	}
-	else if (strncmp (info->devname, "pty/", 4) == 0)
-	{
-	    int index = atoi (info->devname + 5);
-	    const char *pty1 = "pqrstuvwxyzabcde";
-	    const char *pty2 = "0123456789abcdef";
-
-	    sprintf (compat_buf, "%cty%c%c",
-		     (info->devname[4] == 'm') ? 'p' : 't',
-		     pty1[index >> 4], pty2[index & 0x0f]);
-	    compat_name = compat_buf;
-	}
-	else if (strncmp (info->devname, "joysticks/", 10) == 0)
-	{
-	    int index;
-	    char *type = "";
-
-	    if (info->devname[10] == 'a') index = atoi (info->devname + 18);
-	    else
-	    {
-		index = atoi (info->devname + 17);
-		type = "d";
-	    }
-	    sprintf (compat_buf, "%sjs%d", type, index);
-	    compat_name = compat_buf;
-	}
+	compat_name = get_old_name (info->devname, info->namelen, compat_buf,
+				    info->major, info->minor);
 	break;
       case AC_MKNEWCOMPAT:
       case AC_RMNEWCOMPAT:
-	if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-	     (strncmp (ptr = (strrchr (info->devname, '/') + 1), "mt", 2)
-	      == 0) )
+	if (strncmp (info->devname, "scsi/", 5) == 0)
 	{
 	    int mode, host, bus, target, lun;
-	    char rewind = info->devname[info->namelen - 1];
 
-	    if (rewind != 'n') rewind = '\0';
-	    switch (ptr[2])
+	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
+		    &host, &bus, &target, &lun);
+	    compat_name = compat_buf;
+	    sprintf (dest_buf, "../%s", info->devname);
+	    dest_name = dest_buf;
+	    if (strncmp (ptr = (strrchr (info->devname, '/') + 1), "mt", 2)
+		== 0)
 	    {
-	      default:
-		mode = 0;
-		break;
-	      case 'l':
-		mode = 1;
-		break;
-	      case 'm':
-		mode = 2;
-		break;
-	      case 'a':
-		mode = 3;
-		break;
+		char rewind = info->devname[info->namelen - 1];
+
+		if (rewind != 'n') rewind = '\0';
+		switch (ptr[2])
+		{
+		  default:
+		    mode = 0;
+		    break;
+		  case 'l':
+		    mode = 1;
+		    break;
+		  case 'm':
+		    mode = 2;
+		    break;
+		  case 'a':
+		    mode = 3;
+		    break;
+		}
+		sprintf (compat_buf, "st/c%db%dt%du%dm%d%c",
+			 host, bus, target, lun, mode, rewind);
 	    }
-	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
-		    &host, &bus, &target, &lun);
-	    sprintf (compat_buf, "st/c%db%dt%du%dm%d%c",
-		     host, bus, target, lun, mode, rewind);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname);
-	    dest_name = dest_buf;
+	    else if (strcmp (info->devname + info->namelen - 7,"generic") == 0)
+		sprintf (compat_buf, "sg/c%db%dt%du%d",
+			 host, bus, target, lun);
+	    else if (strcmp (info->devname + info->namelen - 2, "cd") == 0)
+		sprintf (compat_buf, "sr/c%db%dt%du%d",
+			 host, bus, target, lun);
+	    else if (strcmp (info->devname + info->namelen - 4, "disc") == 0)
+		sprintf (compat_buf, "sd/c%db%dt%du%d",
+			 host, bus, target, lun);
+	    else if (strncmp (ptr = (strrchr (info->devname, '/') + 1), "part",
+			      4) == 0)
+		sprintf ( compat_buf, "sd/c%db%dt%du%dp%d",
+			  host, bus, target, lun, atoi (ptr + 4) );
+	    else compat_name = NULL;
 	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strcmp (info->devname + info->namelen - 7,"generic") == 0) )
-	{
-	    int host, bus, target, lun;
-
-	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
-		    &host, &bus, &target, &lun);
-	    sprintf (compat_buf, "sg/c%db%dt%du%d", host, bus, target, lun);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname);
-	    dest_name = dest_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strcmp (info->devname + info->namelen - 2, "cd") == 0) )
-	{
-	    int host, bus, target, lun;
-
-	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
-		    &host, &bus, &target, &lun);
-	    sprintf (compat_buf, "sr/c%db%dt%du%d", host, bus, target, lun);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname);
-	    dest_name = dest_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strcmp (info->devname + info->namelen - 4, "disc") == 0) )
-	{
-	    int host, bus, target, lun;
-
-	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/",
-		    &host, &bus, &target, &lun);
-	    sprintf (compat_buf, "sd/c%db%dt%du%d", host, bus, target, lun);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname);
-	    dest_name = dest_buf;
-	}
-	else if ( (strncmp (info->devname, "scsi/", 5) == 0) &&
-		  (strncmp (ptr = (strrchr (info->devname, '/') +1), "part", 4)
-		   == 0) )
-	{
-	    int host, bus, target, lun, partition;
-
-	    sscanf (info->devname + 5, "host%d/bus%d/target%d/lun%d/part%d",
-		    &host, &bus, &target, &lun, &partition);
-	    sprintf (compat_buf, "sd/c%db%dt%du%dp%d",
-		     host, bus, target, lun, partition);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname);
-	    dest_name = dest_buf;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strcmp (info->devname + info->namelen - 4, "disc") == 0) )
+	else if (strncmp (info->devname, "ide/", 4) == 0)
 	{
 	    int host, bus, target, lun;
 
 	    sscanf (info->devname + 4, "host%d/bus%d/target%d/lun%d/",
 		    &host, &bus, &target, &lun);
-	    sprintf (compat_buf, "ide/hd/c%db%dt%du%d",
-		     host, bus, target, lun);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname + 4);
-	    dest_name = dest_buf;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strncmp (ptr = (strrchr (info->devname, '/') +1), "part", 4)
-		   == 0) )
-	{
-	    int host, bus, target, lun, partition;
-
-	    sscanf (info->devname + 4, "host%d/bus%d/target%d/lun%d/part%d",
-		    &host, &bus, &target, &lun, &partition);
-	    sprintf (compat_buf, "ide/hd/c%db%dt%du%dp%d",
-		     host, bus, target, lun, partition);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname + 4);
-	    dest_name = dest_buf;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strcmp (info->devname + info->namelen - 2, "cd") == 0) )
-	{
-	    int host, bus, target, lun;
-
-	    sscanf (info->devname + 4, "host%d/bus%d/target%d/lun%d/",
-		    &host, &bus, &target, &lun);
-	    sprintf (compat_buf, "ide/cd/c%db%dt%du%d", host, bus, target,lun);
-	    compat_name = compat_buf;
-	    sprintf (dest_buf, "../%s", info->devname + 4);
-	    dest_name = dest_buf;
-	}
-	else if ( (strncmp (info->devname, "ide/", 4) == 0) &&
-		  (strncmp (ptr = (strrchr (info->devname, '/') + 1), "mt", 2)
-		   == 0) )
-	{
-	    int host, bus, target, lun;
-
-	    sscanf (info->devname + 4, "host%d/bus%d/target%d/lun%d/",
-		    &host, &bus, &target, &lun);
-	    sprintf (compat_buf, "ide/mt/c%db%dt%du%d%s",
-		     host, bus, target, lun, ptr + 2);
 	    compat_name = compat_buf;
 	    sprintf (dest_buf, "../%s", info->devname + 4);
 	    dest_name = dest_buf;
+	    if (strcmp (info->devname + info->namelen - 4, "disc") == 0)
+		sprintf (compat_buf, "ide/hd/c%db%dt%du%d",
+			 host, bus, target, lun);
+	    else if (strncmp (ptr = (strrchr (info->devname, '/') + 1), "part",
+			      4) == 0)
+		sprintf ( compat_buf, "ide/hd/c%db%dt%du%dp%d",
+			  host, bus, target, lun, atoi (ptr + 4) );
+	    else if (strcmp (info->devname + info->namelen - 2, "cd") == 0)
+		sprintf (compat_buf, "ide/cd/c%db%dt%du%d",
+			 host, bus, target,lun);
+	    else if (strncmp (ptr = (strrchr (info->devname, '/') + 1), "mt",
+			      2) == 0)
+		sprintf (compat_buf, "ide/mt/c%db%dt%du%d%s",
+			 host, bus, target, lun, ptr + 2);
+	    else compat_name = NULL;
 	}
 	break;
     }
